@@ -1,61 +1,73 @@
-# -*- coding: utf-8 -*-
-"""
-    sphinx.ext.napoleon
-    ~~~~~~~~~~~~~~~~~~~
+"""Support for NumPy and Google style docstrings."""
 
-    Support for NumPy and Google style docstrings.
+from typing import Any, Dict, List
 
-    :copyright: Copyright 2007-2015 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
-
-import sys
-
-from six import PY2, iteritems
-
-import sphinx
+from sphinx import __display_version__ as __version__
+from sphinx.application import Sphinx
 from sphinx.ext.napoleon.docstring import GoogleDocstring, NumpyDocstring
+from sphinx.util import inspect
 
 
-class Config(object):
+class Config:
     """Sphinx napoleon extension settings in `conf.py`.
 
     Listed below are all the settings used by napoleon and their default
     values. These settings can be changed in the Sphinx `conf.py` file. Make
-    sure that both "sphinx.ext.autodoc" and "sphinx.ext.napoleon" are
-    enabled in `conf.py`::
+    sure that "sphinx.ext.napoleon" is enabled in `conf.py`::
 
         # conf.py
 
         # Add any Sphinx extension module names here, as strings
-        extensions = ['sphinx.ext.autodoc', 'sphinx.ext.napoleon']
+        extensions = ['sphinx.ext.napoleon']
 
         # Napoleon settings
         napoleon_google_docstring = True
         napoleon_numpy_docstring = True
+        napoleon_include_init_with_doc = False
         napoleon_include_private_with_doc = False
-        napoleon_include_special_with_doc = True
+        napoleon_include_special_with_doc = False
         napoleon_use_admonition_for_examples = False
         napoleon_use_admonition_for_notes = False
         napoleon_use_admonition_for_references = False
         napoleon_use_ivar = False
         napoleon_use_param = True
         napoleon_use_rtype = True
+        napoleon_use_keyword = True
+        napoleon_preprocess_types = False
+        napoleon_type_aliases = None
+        napoleon_custom_sections = None
+        napoleon_attr_annotations = True
 
     .. _Google style:
-       http://google-styleguide.googlecode.com/svn/trunk/pyguide.html
+       https://google.github.io/styleguide/pyguide.html
     .. _NumPy style:
        https://github.com/numpy/numpy/blob/master/doc/HOWTO_DOCUMENT.rst.txt
 
     Attributes
     ----------
-    napoleon_google_docstring : bool, defaults to True
+    napoleon_google_docstring : :obj:`bool` (Defaults to True)
         True to parse `Google style`_ docstrings. False to disable support
         for Google style docstrings.
-    napoleon_numpy_docstring : bool, defaults to True
+    napoleon_numpy_docstring : :obj:`bool` (Defaults to True)
         True to parse `NumPy style`_ docstrings. False to disable support
         for NumPy style docstrings.
-    napoleon_include_private_with_doc : bool, defaults to False
+    napoleon_include_init_with_doc : :obj:`bool` (Defaults to False)
+        True to list ``__init___`` docstrings separately from the class
+        docstring. False to fall back to Sphinx's default behavior, which
+        considers the ``__init___`` docstring as part of the class
+        documentation.
+
+        **If True**::
+
+            def __init__(self):
+                \"\"\"
+                This will be included in the docs because it has a docstring
+                \"\"\"
+
+            def __init__(self):
+                # This will NOT be included in the docs
+
+    napoleon_include_private_with_doc : :obj:`bool` (Defaults to False)
         True to include private members (like ``_membername``) with docstrings
         in the documentation. False to fall back to Sphinx's default behavior.
 
@@ -71,7 +83,7 @@ class Config(object):
                 # This will NOT be included in the docs
                 pass
 
-    napoleon_include_special_with_doc : bool, defaults to True
+    napoleon_include_special_with_doc : :obj:`bool` (Defaults to False)
         True to include special members (like ``__membername__``) with
         docstrings in the documentation. False to fall back to Sphinx's
         default behavior.
@@ -88,7 +100,7 @@ class Config(object):
                 # This will NOT be included in the docs
                 return unicode(self.__class__.__name__)
 
-    napoleon_use_admonition_for_examples : bool, defaults to False
+    napoleon_use_admonition_for_examples : :obj:`bool` (Defaults to False)
         True to use the ``.. admonition::`` directive for the **Example** and
         **Examples** sections. False to use the ``.. rubric::`` directive
         instead. One may look better than the other depending on what HTML
@@ -112,7 +124,7 @@ class Config(object):
 
             This is just a quick example
 
-    napoleon_use_admonition_for_notes : bool, defaults to False
+    napoleon_use_admonition_for_notes : :obj:`bool` (Defaults to False)
         True to use the ``.. admonition::`` directive for **Notes** sections.
         False to use the ``.. rubric::`` directive instead.
 
@@ -125,7 +137,7 @@ class Config(object):
         --------
         :attr:`napoleon_use_admonition_for_examples`
 
-    napoleon_use_admonition_for_references : bool, defaults to False
+    napoleon_use_admonition_for_references : :obj:`bool` (Defaults to False)
         True to use the ``.. admonition::`` directive for **References**
         sections. False to use the ``.. rubric::`` directive instead.
 
@@ -133,7 +145,7 @@ class Config(object):
         --------
         :attr:`napoleon_use_admonition_for_examples`
 
-    napoleon_use_ivar : bool, defaults to False
+    napoleon_use_ivar : :obj:`bool` (Defaults to False)
         True to use the ``:ivar:`` role for instance variables. False to use
         the ``.. attribute::`` directive instead.
 
@@ -153,11 +165,11 @@ class Config(object):
 
             .. attribute:: attr1
 
-               *int*
-
                Description of `attr1`
 
-    napoleon_use_param : bool, defaults to True
+               :type: int
+
+    napoleon_use_param : :obj:`bool` (Defaults to True)
         True to use a ``:param:`` role for each function parameter. False to
         use a single ``:parameters:`` role for all the parameters.
 
@@ -184,7 +196,22 @@ class Config(object):
                          * **arg2** (*int, optional*) --
                            Description of `arg2`, defaults to 0
 
-    napoleon_use_rtype : bool, defaults to True
+    napoleon_use_keyword : :obj:`bool` (Defaults to True)
+        True to use a ``:keyword:`` role for each function keyword argument.
+        False to use a single ``:keyword arguments:`` role for all the
+        keywords.
+
+        This behaves similarly to  :attr:`napoleon_use_param`. Note unlike
+        docutils, ``:keyword:`` and ``:param:`` will not be treated the same
+        way - there will be a separate "Keyword Arguments" section, rendered
+        in the same fashion as "Parameters" section (type links created if
+        possible)
+
+        See Also
+        --------
+        :attr:`napoleon_use_param`
+
+    napoleon_use_rtype : :obj:`bool` (Defaults to True)
         True to use the ``:rtype:`` role for the return type. False to output
         the return type inline with the description.
 
@@ -204,28 +231,62 @@ class Config(object):
 
             :returns: *bool* -- True if successful, False otherwise
 
+    napoleon_preprocess_types : :obj:`bool` (Defaults to False)
+        Enable the type preprocessor.
+
+    napoleon_type_aliases : :obj:`dict` (Defaults to None)
+        Add a mapping of strings to string, translating types in numpy
+        style docstrings. Only works if ``napoleon_preprocess_types = True``.
+
+    napoleon_custom_sections : :obj:`list` (Defaults to None)
+        Add a list of custom sections to include, expanding the list of parsed sections.
+
+        The entries can either be strings or tuples, depending on the intention:
+          * To create a custom "generic" section, just pass a string.
+          * To create an alias for an existing section, pass a tuple containing the
+            alias name and the original, in that order.
+          * To create a custom section that displays like the parameters or returns
+            section, pass a tuple containing the custom section name and a string
+            value, "params_style" or "returns_style".
+
+        If an entry is just a string, it is interpreted as a header for a generic
+        section. If the entry is a tuple/list/indexed container, the first entry
+        is the name of the section, the second is the section key to emulate. If the
+        second entry value is "params_style" or "returns_style", the custom section
+        will be displayed like the parameters section or returns section.
+
+    napoleon_attr_annotations : :obj:`bool` (Defaults to True)
+        Use the type annotations of class attributes that are documented in the docstring
+        but do not have a type in the docstring.
+
     """
     _config_values = {
         'napoleon_google_docstring': (True, 'env'),
         'napoleon_numpy_docstring': (True, 'env'),
+        'napoleon_include_init_with_doc': (False, 'env'),
         'napoleon_include_private_with_doc': (False, 'env'),
-        'napoleon_include_special_with_doc': (True, 'env'),
+        'napoleon_include_special_with_doc': (False, 'env'),
         'napoleon_use_admonition_for_examples': (False, 'env'),
         'napoleon_use_admonition_for_notes': (False, 'env'),
         'napoleon_use_admonition_for_references': (False, 'env'),
         'napoleon_use_ivar': (False, 'env'),
         'napoleon_use_param': (True, 'env'),
         'napoleon_use_rtype': (True, 'env'),
+        'napoleon_use_keyword': (True, 'env'),
+        'napoleon_preprocess_types': (False, 'env'),
+        'napoleon_type_aliases': (None, 'env'),
+        'napoleon_custom_sections': (None, 'env'),
+        'napoleon_attr_annotations': (True, 'env'),
     }
 
-    def __init__(self, **settings):
-        for name, (default, rebuild) in iteritems(self._config_values):
+    def __init__(self, **settings: Any) -> None:
+        for name, (default, _rebuild) in self._config_values.items():
             setattr(self, name, default)
-        for name, value in iteritems(settings):
+        for name, value in settings.items():
             setattr(self, name, value)
 
 
-def setup(app):
+def setup(app: Sphinx) -> Dict[str, Any]:
     """Sphinx extension setup function.
 
     When the extension is loaded, Sphinx imports this module and executes
@@ -239,27 +300,50 @@ def setup(app):
 
     See Also
     --------
-    The Sphinx documentation on `Extensions`_, the `Extension Tutorial`_, and
-    the `Extension API`_.
+    `The Sphinx documentation on Extensions
+    <https://www.sphinx-doc.org/extensions.html>`_
 
-    .. _Extensions: http://sphinx-doc.org/extensions.html
-    .. _Extension Tutorial: http://sphinx-doc.org/ext/tutorial.html
-    .. _Extension API: http://sphinx-doc.org/ext/appapi.html
+    `The Extension Tutorial <https://www.sphinx-doc.org/extdev/tutorial.html>`_
+
+    `The Extension API <https://www.sphinx-doc.org/extdev/appapi.html>`_
 
     """
-    from sphinx.application import Sphinx
     if not isinstance(app, Sphinx):
-        return  # probably called by tests
+        # probably called by tests
+        return {'version': __version__, 'parallel_read_safe': True}
 
+    _patch_python_domain()
+
+    app.setup_extension('sphinx.ext.autodoc')
     app.connect('autodoc-process-docstring', _process_docstring)
     app.connect('autodoc-skip-member', _skip_member)
 
-    for name, (default, rebuild) in iteritems(Config._config_values):
+    for name, (default, rebuild) in Config._config_values.items():
         app.add_config_value(name, default, rebuild)
-    return {'version': sphinx.__version__, 'parallel_read_safe': True}
+    return {'version': __version__, 'parallel_read_safe': True}
 
 
-def _process_docstring(app, what, name, obj, options, lines):
+def _patch_python_domain() -> None:
+    try:
+        from sphinx.domains.python import PyTypedField
+    except ImportError:
+        pass
+    else:
+        import sphinx.domains.python
+        from sphinx.locale import _
+        for doc_field in sphinx.domains.python.PyObject.doc_field_types:
+            if doc_field.name == 'parameter':
+                doc_field.names = ('param', 'parameter', 'arg', 'argument')
+                break
+        sphinx.domains.python.PyObject.doc_field_types.append(
+            PyTypedField('keyword', label=_('Keyword Arguments'),
+                         names=('keyword', 'kwarg', 'kwparam'),
+                         typerolename='obj', typenames=('paramtype', 'kwtype'),
+                         can_collapse=True))
+
+
+def _process_docstring(app: Sphinx, what: str, name: str, obj: Any,
+                       options: Any, lines: List[str]) -> None:
     """Process the docstring for a given python object.
 
     Called when autodoc has read and processed a docstring. `lines` is a list
@@ -296,6 +380,7 @@ def _process_docstring(app, what, name, obj, options, lines):
 
     """
     result_lines = lines
+    docstring: GoogleDocstring = None
     if app.config.napoleon_numpy_docstring:
         docstring = NumpyDocstring(result_lines, app.config, app, what, name,
                                    obj, options)
@@ -307,12 +392,15 @@ def _process_docstring(app, what, name, obj, options, lines):
     lines[:] = result_lines[:]
 
 
-def _skip_member(app, what, name, obj, skip, options):
+def _skip_member(app: Sphinx, what: str, name: str, obj: Any,
+                 skip: bool, options: Any) -> bool:
     """Determine if private and special class members are included in docs.
 
     The following settings in conf.py determine if private and special class
-    members are included in the generated documentation:
+    members or init methods are included in the generated documentation:
 
+    * ``napoleon_include_init_with_doc`` --
+      include init methods if they have docstrings
     * ``napoleon_include_private_with_doc`` --
       include private members if they have docstrings
     * ``napoleon_include_special_with_doc`` --
@@ -348,44 +436,42 @@ def _skip_member(app, what, name, obj, skip, options):
 
     """
     has_doc = getattr(obj, '__doc__', False)
-    is_member = (what == 'class' or what == 'exception' or what == 'module')
-    if name != '__weakref__' and name != '__init__' and has_doc and is_member:
+    is_member = what in ('class', 'exception', 'module')
+    if name != '__weakref__' and has_doc and is_member:
         cls_is_owner = False
-        if what == 'class' or what == 'exception':
-            if PY2:
-                cls = getattr(obj, 'im_class', getattr(obj, '__objclass__',
-                              None))
-                cls_is_owner = (cls and hasattr(cls, name) and
-                                name in cls.__dict__)
-            elif sys.version_info >= (3, 3):
-                qualname = getattr(obj, '__qualname__', '')
-                cls_path, _, _ = qualname.rpartition('.')
-                if cls_path:
-                    try:
-                        if '.' in cls_path:
-                            import importlib
-                            import functools
+        if what in ('class', 'exception'):
+            qualname = getattr(obj, '__qualname__', '')
+            cls_path, _, _ = qualname.rpartition('.')
+            if cls_path:
+                try:
+                    if '.' in cls_path:
+                        import functools
+                        import importlib
 
-                            mod = importlib.import_module(obj.__module__)
-                            mod_path = cls_path.split('.')
-                            cls = functools.reduce(getattr, mod_path, mod)
-                        else:
-                            cls = obj.__globals__[cls_path]
-                    except:
-                        cls_is_owner = False
+                        mod = importlib.import_module(obj.__module__)
+                        mod_path = cls_path.split('.')
+                        cls = functools.reduce(getattr, mod_path, mod)
                     else:
-                        cls_is_owner = (cls and hasattr(cls, name) and
-                                        name in cls.__dict__)
-                else:
+                        cls = inspect.unwrap(obj).__globals__[cls_path]
+                except Exception:
                     cls_is_owner = False
+                else:
+                    cls_is_owner = (cls and hasattr(cls, name) and  # type: ignore
+                                    name in cls.__dict__)
             else:
-                cls_is_owner = True
+                cls_is_owner = False
 
         if what == 'module' or cls_is_owner:
-            is_special = name.startswith('__') and name.endswith('__')
-            is_private = not is_special and name.startswith('_')
+            is_init = (name == '__init__')
+            is_special = (not is_init and name.startswith('__') and
+                          name.endswith('__'))
+            is_private = (not is_init and not is_special and
+                          name.startswith('_'))
+            inc_init = app.config.napoleon_include_init_with_doc
             inc_special = app.config.napoleon_include_special_with_doc
             inc_private = app.config.napoleon_include_private_with_doc
-            if (is_special and inc_special) or (is_private and inc_private):
+            if ((is_special and inc_special) or
+                    (is_private and inc_private) or
+                    (is_init and inc_init)):
                 return False
-    return skip
+    return None
