@@ -1,61 +1,50 @@
-"""
-    sphinx.writers.text
-    ~~~~~~~~~~~~~~~~~~~
-
-    Custom docutils writer for plain text.
-
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
+"""Custom docutils writer for plain text."""
 import math
 import os
 import re
 import textwrap
-import warnings
-from itertools import groupby, chain
-from typing import Callable, Dict, List, Iterable, Optional, Set, Tuple, Union
-from typing import cast
+from itertools import chain, groupby
+from typing import (TYPE_CHECKING, Any, Dict, Generator, Iterable, List, Optional, Set, Tuple,
+                    Union, cast)
 
 from docutils import nodes, writers
-from docutils.nodes import Element, Node, Text
+from docutils.nodes import Element, Text
 from docutils.utils import column_width
 
 from sphinx import addnodes
-from sphinx.deprecation import RemovedInSphinx30Warning
-from sphinx.locale import admonitionlabels, _
+from sphinx.locale import _, admonitionlabels
 from sphinx.util.docutils import SphinxTranslator
 
-if False:
-    # For type annotation
+if TYPE_CHECKING:
     from sphinx.builders.text import TextBuilder
 
 
 class Cell:
     """Represents a cell in a table.
-    It can span on multiple columns or on multiple lines.
+    It can span multiple columns or multiple lines.
     """
-    def __init__(self, text="", rowspan=1, colspan=1):
+    def __init__(self, text: str = "", rowspan: int = 1, colspan: int = 1) -> None:
         self.text = text
-        self.wrapped = []  # type: List[str]
+        self.wrapped: List[str] = []
         self.rowspan = rowspan
         self.colspan = colspan
-        self.col = None
-        self.row = None
+        self.col: Optional[int] = None
+        self.row: Optional[int] = None
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return "<Cell {!r} {}v{}/{}>{}>".format(
             self.text, self.row, self.rowspan, self.col, self.colspan
         )
 
-    def __hash__(self):
+    def __hash__(self) -> int:
         return hash((self.col, self.row))
 
-    def wrap(self, width):
+    def wrap(self, width: int) -> None:
         self.wrapped = my_wrap(self.text, width)
 
 
 class Table:
-    """Represents a table, handling cells that can span on multiple lines
+    """Represents a table, handling cells that can span multiple lines
     or rows, like::
 
        +-----------+-----+
@@ -66,22 +55,22 @@ class Table:
        | DDD | CCC       |
        +-----+-----------+
 
-    This class can be used in two ways:
+    This class can be used in two ways, either:
 
-    - Either with absolute positions: call ``table[line, col] = Cell(...)``,
-      this overwrite an existing cell if any.
+    - With absolute positions: call ``table[line, col] = Cell(...)``,
+      this overwrites any existing cell(s) at these positions.
 
-    - Either with relative positions: call the ``add_row()`` and
+    - With relative positions: call the ``add_row()`` and
       ``add_cell(Cell(...))`` as needed.
 
-    Cell spanning on multiple rows or multiple columns (having a
+    Cells spanning multiple rows or multiple columns (having a
     colspan or rowspan greater than one) are automatically referenced
-    by all the table cells they covers. This is a usefull
-    representation as we can simply check ``if self[x, y] is self[x,
-    y+1]`` to recognize a rowspan.
+    by all the table cells they cover. This is a useful
+    representation as we can simply check
+    ``if self[x, y] is self[x, y+1]`` to recognize a rowspan.
 
     Colwidth is not automatically computed, it has to be given, either
-    at construction time, either during the table construction.
+    at construction time, or during the table construction.
 
     Example usage::
 
@@ -100,30 +89,28 @@ class Table:
        +--------+--------+
 
     """
-    def __init__(self, colwidth=None):
-        self.lines = []  # type: List[List[Cell]]
+    def __init__(self, colwidth: List[int] = None) -> None:
+        self.lines: List[List[Cell]] = []
         self.separator = 0
-        self.colwidth = (colwidth if colwidth is not None
-                         else [])  # type: List[int]
+        self.colwidth: List[int] = (colwidth if colwidth is not None else [])
         self.current_line = 0
         self.current_col = 0
 
-    def add_row(self):
+    def add_row(self) -> None:
         """Add a row to the table, to use with ``add_cell()``.  It is not needed
         to call ``add_row()`` before the first ``add_cell()``.
         """
         self.current_line += 1
         self.current_col = 0
 
-    def set_separator(self):
-        """Sets the separator below the current line.
-        """
+    def set_separator(self) -> None:
+        """Sets the separator below the current line."""
         self.separator = len(self.lines)
 
-    def add_cell(self, cell):
+    def add_cell(self, cell: Cell) -> None:
         """Add a cell to the current line, to use with ``add_row()``.  To add
-        a cell spanning on multiple lines or rows, simply set the
-        ``cell.colspan`` or ``cell.rowspan`` BEFORE inserting it to
+        a cell spanning multiple lines or rows, simply set the
+        ``cell.colspan`` or ``cell.rowspan`` BEFORE inserting it into
         the table.
         """
         while self[self.current_line, self.current_col]:
@@ -131,13 +118,13 @@ class Table:
         self[self.current_line, self.current_col] = cell
         self.current_col += cell.colspan
 
-    def __getitem__(self, pos):
+    def __getitem__(self, pos: Tuple[int, int]) -> Cell:
         line, col = pos
         self._ensure_has_line(line + 1)
         self._ensure_has_column(col + 1)
         return self.lines[line][col]
 
-    def __setitem__(self, pos, cell):
+    def __setitem__(self, pos: Tuple[int, int], cell: Cell) -> None:
         line, col = pos
         self._ensure_has_line(line + cell.rowspan)
         self._ensure_has_column(col + cell.colspan)
@@ -147,22 +134,22 @@ class Table:
                 cell.row = line
                 cell.col = col
 
-    def _ensure_has_line(self, line):
+    def _ensure_has_line(self, line: int) -> None:
         while len(self.lines) < line:
             self.lines.append([])
 
-    def _ensure_has_column(self, col):
+    def _ensure_has_column(self, col: int) -> None:
         for line in self.lines:
             while len(line) < col:
                 line.append(None)
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return "\n".join(repr(line) for line in self.lines)
 
-    def cell_width(self, cell, source):
+    def cell_width(self, cell: Cell, source: List[int]) -> int:
         """Give the cell width, according to the given source (either
         ``self.colwidth`` or ``self.measured_widths``).
-        This take into account cells spanning on multiple columns.
+        This takes into account cells spanning multiple columns.
         """
         width = 0
         for i in range(self[cell.row, cell.col].colspan):
@@ -170,15 +157,15 @@ class Table:
         return width + (cell.colspan - 1) * 3
 
     @property
-    def cells(self):
-        seen = set()  # type: Set[Cell]
-        for lineno, line in enumerate(self.lines):
-            for colno, cell in enumerate(line):
+    def cells(self) -> Generator[Cell, None, None]:
+        seen: Set[Cell] = set()
+        for line in self.lines:
+            for cell in line:
                 if cell and cell not in seen:
                     yield cell
                     seen.add(cell)
 
-    def rewrap(self):
+    def rewrap(self) -> None:
         """Call ``cell.wrap()`` on all cells, and measure each column width
         after wrapping (result written in ``self.measured_widths``).
         """
@@ -191,8 +178,8 @@ class Table:
             for col in range(cell.col, cell.col + cell.colspan):
                 self.measured_widths[col] = max(self.measured_widths[col], width)
 
-    def physical_lines_for_line(self, line):
-        """From a given line, compute the number of physical lines it spans
+    def physical_lines_for_line(self, line: List[Cell]) -> int:
+        """For a given line, compute the number of physical lines it spans
         due to text wrapping.
         """
         physical_lines = 1
@@ -200,7 +187,7 @@ class Table:
             physical_lines = max(physical_lines, len(cell.wrapped))
         return physical_lines
 
-    def __str__(self):
+    def __str__(self) -> str:
         out = []
         self.rewrap()
 
@@ -208,7 +195,7 @@ class Table:
             """Called on the line *before* lineno.
             Called with no *lineno* for the last sep.
             """
-            out = []  # type: List[str]
+            out: List[str] = []
             for colno, width in enumerate(self.measured_widths):
                 if (
                     lineno is not None and
@@ -225,7 +212,7 @@ class Table:
                 for left, right in zip(out, out[1:])
             ]
             glue.append(tail)
-            return head + "".join(chain(*zip(out, glue)))
+            return head + "".join(chain.from_iterable(zip(out, glue)))
 
         for lineno, line in enumerate(self.lines):
             if self.separator and lineno == self.separator:
@@ -270,7 +257,7 @@ class TextWrapper(textwrap.TextWrapper):
         The original _wrap_chunks uses len() to calculate width.
         This method respects wide/fullwidth characters for width adjustment.
         """
-        lines = []  # type: List[str]
+        lines: List[str] = []
         if self.width <= 0:
             raise ValueError("invalid width %r (must be > 0)" % self.width)
 
@@ -327,11 +314,11 @@ class TextWrapper(textwrap.TextWrapper):
         """_split(text : string) -> [string]
 
         Override original method that only split by 'wordsep_re'.
-        This '_split' split wide-characters into chunk by one character.
+        This '_split' splits wide-characters into chunks by one character.
         """
         def split(t: str) -> List[str]:
             return super(TextWrapper, self)._split(t)
-        chunks = []  # type: List[str]
+        chunks: List[str] = []
         for chunk in split(text):
             for w, g in groupby(chunk, column_width):
                 if w == 1:
@@ -362,7 +349,7 @@ MAXWIDTH = 70
 STDINDENT = 3
 
 
-def my_wrap(text: str, width: int = MAXWIDTH, **kwargs) -> List[str]:
+def my_wrap(text: str, width: int = MAXWIDTH, **kwargs: Any) -> List[str]:
     w = TextWrapper(width=width, **kwargs)
     return w.wrap(text)
 
@@ -370,9 +357,9 @@ def my_wrap(text: str, width: int = MAXWIDTH, **kwargs) -> List[str]:
 class TextWriter(writers.Writer):
     supported = ('text',)
     settings_spec = ('No options here.', '', ())
-    settings_defaults = {}  # type: Dict
+    settings_defaults: Dict = {}
 
-    output = None  # type: str
+    output: str = None
 
     def __init__(self, builder: "TextBuilder") -> None:
         super().__init__()
@@ -385,7 +372,7 @@ class TextWriter(writers.Writer):
 
 
 class TextTranslator(SphinxTranslator):
-    builder = None  # type: TextBuilder
+    builder: "TextBuilder" = None
 
     def __init__(self, document: nodes.document, builder: "TextBuilder") -> None:
         super().__init__(document, builder)
@@ -400,12 +387,12 @@ class TextTranslator(SphinxTranslator):
         self.sectionchars = self.config.text_sectionchars
         self.add_secnumbers = self.config.text_add_secnumbers
         self.secnumber_suffix = self.config.text_secnumber_suffix
-        self.states = [[]]      # type: List[List[Tuple[int, Union[str, List[str]]]]]
+        self.states: List[List[Tuple[int, Union[str, List[str]]]]] = [[]]
         self.stateindent = [0]
-        self.list_counter = []  # type: List[int]
+        self.list_counter: List[int] = []
         self.sectionlevel = 0
         self.lineblocklevel = 0
-        self.table = None       # type: Table
+        self.table: Table = None
 
     def add_text(self, text: str) -> None:
         self.states[-1].append((-1, text))
@@ -418,8 +405,8 @@ class TextTranslator(SphinxTranslator):
         content = self.states.pop()
         maxindent = sum(self.stateindent)
         indent = self.stateindent.pop()
-        result = []     # type: List[Tuple[int, List[str]]]
-        toformat = []   # type: List[str]
+        result: List[Tuple[int, List[str]]] = []
+        toformat: List[str] = []
 
         def do_format() -> None:
             if not toformat:
@@ -540,6 +527,13 @@ class TextTranslator(SphinxTranslator):
     def depart_attribution(self, node: Element) -> None:
         pass
 
+    #############################################################
+    # Domain-specific object descriptions
+    #############################################################
+
+    # Top-level nodes
+    #################
+
     def visit_desc(self, node: Element) -> None:
         pass
 
@@ -559,6 +553,22 @@ class TextTranslator(SphinxTranslator):
     def depart_desc_signature_line(self, node: Element) -> None:
         self.add_text('\n')
 
+    def visit_desc_content(self, node: Element) -> None:
+        self.new_state()
+        self.add_text(self.nl)
+
+    def depart_desc_content(self, node: Element) -> None:
+        self.end_state()
+
+    def visit_desc_inline(self, node: Element) -> None:
+        pass
+
+    def depart_desc_inline(self, node: Element) -> None:
+        pass
+
+    # Nodes for high-level structure in signatures
+    ##############################################
+
     def visit_desc_name(self, node: Element) -> None:
         pass
 
@@ -610,12 +620,7 @@ class TextTranslator(SphinxTranslator):
     def depart_desc_annotation(self, node: Element) -> None:
         pass
 
-    def visit_desc_content(self, node: Element) -> None:
-        self.new_state()
-        self.add_text(self.nl)
-
-    def depart_desc_content(self, node: Element) -> None:
-        self.end_state()
+    ##############################################
 
     def visit_figure(self, node: Element) -> None:
         self.new_state()
@@ -837,7 +842,7 @@ class TextTranslator(SphinxTranslator):
             self.end_state(first='%s. ' % self.list_counter[-1])
 
     def visit_definition_list_item(self, node: Element) -> None:
-        self._classifier_count_in_li = len(node.traverse(nodes.classifier))
+        self._classifier_count_in_li = len(list(node.findall(nodes.classifier)))
 
     def depart_definition_list_item(self, node: Element) -> None:
         pass
@@ -918,7 +923,6 @@ class TextTranslator(SphinxTranslator):
     def _depart_admonition(self, node: Element) -> None:
         label = admonitionlabels[node.tagname]
         indent = sum(self.stateindent) + len(label)
-        print(self.states[-1])
         if (len(self.states[-1]) == 1 and
                 self.states[-1][0][0] == 0 and
                 MAXWIDTH - indent >= sum(len(s) for s in self.states[-1][0][1])):
@@ -1016,6 +1020,9 @@ class TextTranslator(SphinxTranslator):
     def visit_toctree(self, node: Element) -> None:
         raise nodes.SkipNode
 
+    def visit_substitution_definition(self, node: Element) -> None:
+        raise nodes.SkipNode
+
     def visit_pending_xref(self, node: Element) -> None:
         pass
 
@@ -1174,14 +1181,3 @@ class TextTranslator(SphinxTranslator):
 
     def depart_math_block(self, node: Element) -> None:
         self.end_state()
-
-    def unknown_visit(self, node: Node) -> None:
-        raise NotImplementedError('Unknown node: ' + node.__class__.__name__)
-
-    def _make_depart_admonition(name: str) -> Callable[["TextTranslator", Element], None]:  # type: ignore  # NOQA
-        warnings.warn('TextTranslator._make_depart_admonition() is deprecated.',
-                      RemovedInSphinx30Warning)
-
-        def depart_admonition(self, node: Element) -> None:
-            self.end_state(first=admonitionlabels[name] + ': ')
-        return depart_admonition
