@@ -1,13 +1,6 @@
-"""
-    test_domain_cpp
-    ~~~~~~~~~~~~~~~
-
-    Tests the C++ Domain
-
-    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
+"""Tests the C++ Domain"""
 
+import itertools
 import re
 import zlib
 
@@ -123,7 +116,9 @@ def test_domain_cpp_ast_fundamental_types():
         def makeIdV1():
             if t == 'decltype(auto)':
                 return None
-            id = t.replace(" ", "-").replace("long", "l").replace("int", "i")
+            id = t.replace(" ", "-").replace("long", "l")
+            if "__int" not in t:
+                id = id.replace("int", "i")
             id = id.replace("bool", "b").replace("char", "c")
             id = id.replace("wc_t", "wchar_t").replace("c16_t", "char16_t")
             id = id.replace("c8_t", "char8_t")
@@ -135,7 +130,17 @@ def test_domain_cpp_ast_fundamental_types():
             if t == "std::nullptr_t":
                 id = "NSt9nullptr_tE"
             return "1f%s" % id
-        check("function", "void f(%s arg)" % t, {1: makeIdV1(), 2: makeIdV2()})
+        id1 = makeIdV1()
+        id2 = makeIdV2()
+        input = "void f(%s arg)" % t.replace(' ', '  ')
+        output = "void f(%s arg)" % t
+        check("function", input, {1: id1, 2: id2}, output=output)
+        if ' ' in t:
+            # try permutations of all components
+            tcs = t.split()
+            for p in itertools.permutations(tcs):
+                input = "void f(%s arg)" % ' '.join(p)
+                check("function", input, {1: id1, 2: id2})
 
 
 def test_domain_cpp_ast_expressions():
@@ -635,6 +640,9 @@ def test_domain_cpp_ast_function_definitions():
     # from #8960
     check('function', 'void f(void (*p)(int, double), int i)', {2: '1fPFvidEi'})
 
+    # from #9535 comment
+    check('function', 'void f(void (*p)(int) = &foo)', {2: '1fPFviE'})
+
 
 def test_domain_cpp_ast_operators():
     check('function', 'void operator new()', {1: "new-operator", 2: "nwv"})
@@ -980,6 +988,11 @@ def test_domain_cpp_ast_attributes():
     # position: parameters and qualifiers
     check('function', 'void f() [[attr1]] [[attr2]]', {1: 'f', 2: '1fv'})
 
+    # position: class, union, enum
+    check('class', '{key}[[nodiscard]] Foo', {1: 'Foo', 2: '3Foo'}, key='class')
+    check('union', '{key}[[nodiscard]] Foo', {1: None, 2: '3Foo'}, key='union')
+    check('enum', '{key}[[nodiscard]] Foo', {1: None, 2: '3Foo'}, key='enum')
+
 
 def test_domain_cpp_ast_xref_parsing():
     def check(target):
